<?php declare(strict_types = 1);

namespace Drupal\trucie\Importer;

use Drupal\Core\Entity\EntityInterface;

/**
 * Provides batch entity import from csv file.
 */
class CsvBatchImporter extends BatchImporterBase {

  /**
   * {@inheritDoc}
   */
  protected function readSource(): self {
    $file = $this->loadFile();
    $num = 0;

    if ($file && ($handle = fopen($file->getFileUri(), 'r'))) {
      while (($row = fgetcsv($handle, 0, $this->sourceParams['delimiter'] ?: ';', $this->sourceParams['enclosure'] ?? '"')) !== FALSE) {
        $row['#trucie_row_num'] = ++$num === 1 ? '#trucie_row_num' : $num;
        $this->addOperation($row);
      }

      fclose($handle);
    }

    if (!$num) {
      $this->getMessenger()->addError($this->t('File @file is empty or not readable.', [
        '@file' => $file ? $file->getFilename() : "[fid:{$this->fid}]",
      ]));
    }

    return $this;
  }

  /**
   * {@inheritDoc}
   */
  public function processItem(array $data, array &$context): ?EntityInterface {
    $storage = $this->getEntityTypeManager()->getStorage($this->getImportEntityType());
    $uniqueField = $this->getUniqueField();
    $mode = $this->getImportMode();
    $meta['row_num'] = $data['#trucie_row_num'];
    $op = 'skip';

    if (!empty($data['#trucie_skip_row'])) {
      $context['results'][$op] = ($context['results'][$op] ?? 0) + 1;
      $this->logError(
        $data['#trucie_skip_row'],
        $meta + ['op' => $op]
      );
      return NULL;
    }

    if (!trim((string) ($data[$uniqueField] ?? ''))) {
      $context['results'][$op] = ($context['results'][$op] ?? 0) + 1;
      $this->logError(
        'The unique field is empty.',
        $meta + ['op' => $op]
      );
      return NULL;
    }

    $entity = $storage->loadByProperties([$uniqueField => $data[$uniqueField]]) ?: [];
    $entity = reset($entity);
    $exists = !empty($entity);

    if ($entity && $mode === 'create') {
      $context['results'][$op] = ($context['results'][$op] ?? 0) + 1;
      $this->logStatus(
        "The entity already exists (id={$entity->id()}).",
        $meta + ['op' => $op]
      );
      return NULL;
    }

    $entity = $this->updateEntityFromArray($data, $entity ?: NULL, $storage);
    $op = $exists ? 'update' : 'create';

    $message = "id={$entity->id()}: {$entity->label()}";
    $context['results'][$op] = ($context['results'][$op] ?? 0) + 1;
    $context['message'] = $entity->label();
    $this->logStatus(
      $message,
      $meta + ['op' => $op]
    );

    return $entity;
  }

}
